// If this code works, it was written by Svetlio Kapralov. If not, I don't know who wrote it.
package net.playerfinder.jsf.components.colorpicker;

import java.io.IOException;
import java.util.List;
import java.util.Map;

import javax.faces.component.UIComponent;
import javax.faces.component.behavior.ClientBehavior;
import javax.faces.component.behavior.ClientBehaviorContext;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;
import javax.faces.render.FacesRenderer;
import javax.faces.render.Renderer;

import net.playerfinder.jsf.components.common.Utils;

@FacesRenderer(componentFamily = UIColorPicker.COMPONENT_FAMILY, rendererType = UIColorPicker.DEFAULT_RENDERER)
public class UIColorPickerRenderer extends Renderer {

	@Override
	public void decode(FacesContext context, UIComponent component) {
		UIColorPicker colorPicker = (UIColorPicker) component;

		String clientId = colorPicker.getClientId();
		String value = context.getExternalContext().getRequestParameterMap()
				.get(clientId + "_input");

		if (value != null)
			colorPicker.setSubmittedValue(value);

		Utils.decodeBehaviors(context, component);
	}

	@Override
	public void encodeEnd(FacesContext context, UIComponent component)
			throws IOException {
		UIColorPicker colorPicker = (UIColorPicker) component;

		boolean isAjax = encodeMarkup(context, colorPicker);
		encodeScript(context, colorPicker, isAjax);
	}

	protected boolean encodeMarkup(FacesContext context,
			UIColorPicker colorPicker) throws IOException {
		boolean result = false;
		ResponseWriter writer = context.getResponseWriter();
		String clientId = colorPicker.getClientId(context);
		String inputId = clientId + "_input";
		String value = (String) colorPicker.getValue();
		boolean isPopup = colorPicker.getMode().equals("popup");

		writer.startElement("span", null);
		writer.writeAttribute("id", clientId, "id");

		if (isPopup) {
			encodeButton(context, clientId, value);
		} else {
			encodeInline(context, clientId);
		}

		String onchange = null;
		ClientBehaviorContext behaviorContext = ClientBehaviorContext
				.createClientBehaviorContext(context, colorPicker, "change",
						clientId, null);
		Map<String, List<ClientBehavior>> behaviors = colorPicker
				.getClientBehaviors();
		if (behaviors.containsKey("change")) {
			onchange = behaviors.get("change").get(0)
					.getScript(behaviorContext);
			result = true;
		}

		// Input
		writer.startElement("input", null);
		writer.writeAttribute("id", inputId, "id");
		writer.writeAttribute("name", inputId, "name");
		writer.writeAttribute("type", "hidden", null);
		if (value != null) {
			writer.writeAttribute("value", value, null);
		}

		if (onchange != null)
			writer.writeAttribute("onchange", onchange, null);

		writer.endElement("input");

		writer.endElement("span");

		return result;
	}

	protected void encodeButton(FacesContext context, String clientId,
			String value) throws IOException {

		ResponseWriter writer = context.getResponseWriter();

		writer.startElement("button", null);
		writer.writeAttribute("id", clientId + "_button", null);
		writer.writeAttribute("type", "button", null);

		// text
		writer.startElement("span", null);

		writer.write("<span id=\""
				+ clientId
				+ "_livePreview\" style=\"overflow:hidden;width:1em;height:1em;display:block;border:solid 1px #000;text-indent:1em;white-space:nowrap;");
		if (value != null) {
			writer.write("background-color:#" + value);
		}
		writer.write("\">Live Preview</span>");
		writer.endElement("span");

		writer.endElement("button");
	}

	protected void encodeInline(FacesContext context, String clientId)
			throws IOException {
		ResponseWriter writer = context.getResponseWriter();

		writer.startElement("div", null);
		writer.writeAttribute("id", clientId + "_inline", "id");
		writer.endElement("div");
	}

	protected void encodeScript(FacesContext context,
			UIColorPicker colorPicker, boolean isAjax) throws IOException {
		ResponseWriter writer = context.getResponseWriter();
		String clientId = colorPicker.getClientId(context);
		String value = (String) colorPicker.getValue();

		writer.startElement("script", null);

		String jqueryElementId = "";
		boolean isPopup = colorPicker.getMode().equals("popup");
		if (isPopup)
			jqueryElementId = Utils.escapeIdForJQuery(clientId + "_button");
		else
			jqueryElementId = Utils.escapeIdForJQuery(clientId + "_inline");

		writer.write("pf_jQuery(function($) { " + "$('#" + jqueryElementId
				+ "').ColorPicker( { ");
		writer.write("id:'" + clientId + "'");

		if (!isPopup)
			writer.write(",flat: true");

		if (value != null)
			writer.write(",color:'" + value + "'");

		if (isPopup) {
			writer.write(",onShow: function (colpkr) {");
			writer.write("$(colpkr).fadeIn(300);");
			writer.write("return false; }");
			writer.write(",onHide: function (colpkr) {");
			writer.write("$(colpkr).fadeOut(300);");
			writer.write("return false;}");
		}

		// writer.write(",onChange: function (hsb, hex, rgb) { ");
		writer.write(",onSubmit: function(hsb, hex, rgb, el) { ");
		if (isPopup) {
			writer.write("$('#"
					+ Utils.escapeIdForJQuery(clientId + "_livePreview")
					+ "').css('backgroundColor', '#' + hex); ");
		}
		writer.write("document.getElementById('" + clientId + "_input"
				+ "').value = hex;");
		if (isAjax)
			writer.write("document.getElementById('" + clientId + "_input"
					+ "').onchange();");
		writer.write("}");

		writer.write("}); });");
		writer.endElement("script");
	}
}